var field, left, time, button, down;
var row = 0, col = 0, push = 0, flags = 0, seconds = 0, blanks = 0;
var start = null, stop = null, game = "new";
var rows, cols, mines, marks, graphics;
var grid;

function Cell(img) {
  this.img = img;
}

Cell.prototype.reset = function reset() {
  this.value = 0;
  this.show = null;
  this.pushed = false;
  this.img.setAttribute("index", "c");
}

Cell.prototype.index = function index(index) {
  if (!index) switch (this.show) {
  case "flag": index = "d"; break;
  case "mark": index = this.pushed ? "f" : "e"; break;
  case "true": index = this.value; break;
  default: index = this.pushed ? "0" : "c"; break;
  }
  if (index != this.img.getAttribute("index"))
    this.img.setAttribute("index", index);
}

Cell.prototype.won = function won() {
  if (this.value == 9)
    this.index("d");
}

Cell.prototype.lost = function lost() {
  if (this.value == 9)
    switch (this.show) {
    default: this.index("a");
    case "flag": case "true": return;
  }
  switch (this.show) {
  case "flag": case "mark": this.index("b");
  }
}

function updateFields(pushed) {
  for (var l = Math.max(0, row - push + 1); l < Math.min(rows, row + push); l++) {
    var array = grid[l];
    for (var m = Math.max(0, col - push + 1); m < Math.min(cols, col + push); m++) {
      var cell = array[m];
      cell.pushed = pushed;
      cell.index();
    }
  }
}

function randInt(max) {
  return parseInt(max * Math.random());
}

function tick() {
  if (start) {
    var now = stop || new Date();
    var elapsed = now - start;
    var seconds = parseInt(elapsed / 1000);
    if (!stop) setTimeout(tick, (seconds + 1) * 1000 - elapsed);
    seconds = "00" + seconds;
    seconds = seconds.substring(seconds.length - 3);
    time.childNodes[0].setAttribute("index", seconds[0]);
    time.childNodes[1].setAttribute("index", seconds[1]);
    time.childNodes[2].setAttribute("index", seconds[2]);
  }
}

function startTimer() {
  if (game == "new") {
    game = "play";
    start = new Date();
    setTimeout(tick, 1000);
  }
}

function updateFlags() {
  var diff = mines - flags;
  if (diff < 0) {
    diff = -diff;
    diff = "0" + diff;
    diff = "a" + diff.substring(diff.length - 2);
  } else {
    diff = "00" + diff;
    diff = diff.substring(diff.length - 3);
  }
  left.childNodes[0].setAttribute("index", diff[0]);
  left.childNodes[1].setAttribute("index", diff[1]);
  left.childNodes[2].setAttribute("index", diff[2]);
}

function lost() {
  for (var i = 0; i < rows; i++)
    for (var j = 0; j < cols; j++)
      grid[i][j].lost();
}

function won() {
  for (var i = 0; i < rows; i++)
    for (var j = 0; j < cols; j++)
      grid[i][j].won();
}

function revealField(i, j) {
  var cell = grid[i][j];
  if (!cell.show || cell.show == "mark") {
    cell.show = "true";
    cell.index();
    if (cell.value == 9) {
      stop = new Date();
      game = "lost";
      button.setAttribute("state", "lost");
      setTimeout(lost, 100);
    } else {
      if (cell.value == 0) {
        for (var l = Math.max(0, i - 1); l < Math.min(rows, i + 2); l++)
          for (var m = Math.max(0, j - 1); m < Math.min(cols, j + 2); m++)
            revealField(l, m);
      }
      if (--blanks == 0) {
        stop = new Date();
        game = "won";
        button.setAttribute("state", "won");
        flags = mines;
        updateFlags();
        setTimeout(won, 100);
      }
    }
  }
}

function restart() {
  button.setAttribute("state", "down");
  setTimeout(newGame, 100);
}

function newGame() {
  down = stop = start = null;
  game = "new";
  seconds = 0;
  flags = 0;
  blanks = rows * cols - mines;
  time.childNodes[0].setAttribute("index", 0);
  time.childNodes[1].setAttribute("index", 0);
  time.childNodes[2].setAttribute("index", 0);
  updateFlags();
  for (var i = 0; i < rows; i++) {
    var array = grid[i];
    for (var j = 0; j < cols; j++)
      array[j].reset();
  }
  for (var k = 0; k < mines; k++) {
    do {
      i = randInt(rows);
      j = randInt(cols);
      var cell = grid[i][j];
    } while (cell.value == 9);
    cell.value = 9;
    for (var l = Math.max(0, i - 1); l < Math.min(rows, i + 2); l++) {
      for (var m = Math.max(0, j - 1); m < Math.min(cols, j + 2); m++) {
        cell = grid[l][m];
        if (cell.value != 9)
          cell.value++;
      }
    }
  }
  button.removeAttribute("state");
}

function getIntParam(regexp, min, avg, max) {
  var match = location.href.match(regexp);
  if (match) {
    var val = parseInt(match[1]);                                         
    if (val >= min && val <= max) return val;
  }
  return avg;
}

function getBoolParam(regexp) {
  var match = location.href.match(regexp);
  return (match && match[1] == "true");
}

function onLoad() {
  rows = getIntParam(/[?&]rows=(\d+)/, 8, 16, 24);
  cols = getIntParam(/[?&]cols=(\d+)/, 8, 30, 30);
  mines = getIntParam(/[?&]mines=(\d+)/, 10, (rows * cols * 5) >> 5, (rows - 1) * (cols - 1));
  marks = getBoolParam(/[?&]marks=(\w+)/) ? "mark" : null;
  graphics = getBoolParam(/[?&]graphics=(\w+)/) ? "tield" : "field";
  field = document.getElementById("field");
  left = document.getElementById("left");
  time = document.getElementById("time");
  button = document.getElementById("button");
  field.setAttribute("graphics", graphics);
  grid = new Array(rows);
  var hbox = field.firstChild;
  var img = hbox.firstChild;
  var array = grid[0] = new Array(cols);
  array[0] = new Cell(img);
  for (var j = 1; j < cols; j++) {
    img = img.cloneNode(false);
    img.setAttribute("col", j);
    hbox.appendChild(img);
    array[j] = new Cell(img);
  }
  for (var i = 1; i < rows; i++) {
    hbox = hbox.cloneNode(true);
    hbox.setAttribute("row", i);
    field.appendChild(hbox);
    array = grid[i] = new Array(cols);
    for (j = 0; j < cols; j++)
      array[j] = new Cell(hbox.childNodes[j]);
  }
  newGame();
  try {
    Components.classes;
    document.documentElement.setAttribute("title", document.documentElement.getAttribute("titlemodifier"));
    sizeToContent();
  } catch (e) {
  }
}

function onMouseDown(event) {
  if (!down) {
    if (event.button == 0 && event.target == button) {
      down = button;
      button.setAttribute("state", "down");
    } else if (!stop && event.target.getAttribute("class") == "field") {
      row = parseInt(event.target.parentNode.getAttribute("row"));
      col = parseInt(event.target.getAttribute("col"));
      var cell = grid[row][col];
      if (event.button == 0 || cell.show == "true") {
        down = field;
        button.setAttribute("state", "field");
        push = event.button == 0 ? 1 : 2;
        updateFields(true);
      } else {
        switch (cell.show) {
        case "flag": flags--; cell.show = marks; break;
        case "mark": cell.show = null; break;
        default: flags++; cell.show = "flag"; break;
        }
        cell.index();
        updateFlags();
      }
    }
  } else if (down == field && push == 1) {
    push = 2;
    updateFields(true);
  }
}

function onMouseMove(event) {
  if (down == button) {
    if (event.target == button)
      button.setAttribute("state", "down");
    else
      button.setAttribute("state", game);
  }
  if (down == field) {
    if (event.target.getAttribute("class") == "field") {
      var i = parseInt(event.target.parentNode.getAttribute("row"));
      var j = parseInt(event.target.getAttribute("col"));
    } else {
      i = j = -push;
    }
    if (i != row || j != col) {
      updateFields(false);
      row = i;
      col = j;
      updateFields(true);
    }
  }
}

function onMouseUp(event) {
  if (down == button && event.target == button)
    restart();
  if (down == field) {
    updateFields(false);
    button.removeAttribute("state");
    if (event.target.getAttribute("class") == "field") {
      startTimer();
      row = parseInt(event.target.parentNode.getAttribute("row"));
      col = parseInt(event.target.getAttribute("col"));
      var cell = grid[row][col];
      if (push == 1) {
        revealField(row, col);
      } else if (cell.show == "true") {
        var count = cell.value;
        for (var l = Math.max(0, row - 1); l < Math.min(rows, row + 2); l++)
          for (var m = Math.max(0, col - 1); m < Math.min(cols, col + 2); m++)
            if (grid[l][m].show == "flag")
              count--;
        if (!count)
          for (l = Math.max(0, row - 1); l < Math.min(rows, row + 2); l++)
            for (m = Math.max(0, col - 1); m < Math.min(cols, col + 2); m++)
              revealField(l, m);
      }
    }
  }
  down = null;
}
